import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import {DatePickerComponent} from './date-picker.component';
import {FormsModule} from '@angular/forms';
import {NoopAnimationsModule} from '@angular/platform-browser/animations';
import {Component, SimpleChange, SimpleChanges} from '@angular/core';
import {MatDatepickerModule} from '@angular/material/datepicker';
import {MatMomentDateModule} from '@angular/material-moment-adapter';
import {MatInputModule} from '@angular/material/input';
import {JsonTransformerUtil} from '../../utils/json-transformer.util';

@Component({
  // tslint:disable-next-line:component-selector
  selector: 'mat-icon',
  template: '<p>i</p>'
})
class IconMockComponent {}

describe('DatePickerComponent', () => {
  let component: DatePickerComponent;
  let fixture: ComponentFixture<DatePickerComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [ DatePickerComponent, IconMockComponent],
      imports: [MatDatepickerModule, MatMomentDateModule, MatInputModule, FormsModule, NoopAnimationsModule],
    })
    .compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(DatePickerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('only registers changes if model has structural changes', () => {
    const onChange = jasmine.createSpy('onChange');
    const date = new Date();
    const date2 = JsonTransformerUtil.transformDates(JSON.parse(JSON.stringify(date)));

    component.writeValue(date);
    component.registerOnChange(onChange);
    component.writeValue(date2);
    expect(onChange).toHaveBeenCalledTimes(0);
    component.writeValue( new Date(Date.parse('01-01-2020 00:00:00')));
    expect(onChange).toHaveBeenCalledTimes(1);
  });

  it('detects mouse hover events', (done) => {
    fixture.whenStable().then(() => {
      const element = fixture.elementRef.nativeElement;
      element.dispatchEvent(new Event('mouseenter'));
      fixture.detectChanges();
      expect(component.hover).toBeTrue();

      element.dispatchEvent(new Event('mouseleave'));
      fixture.detectChanges();
      expect(component.hover).toBeFalse();
      done();
    });
  });

  it('registers as touched on blur', (done) => {
    let touched = false;
    component.registerOnTouched(() => touched = true);
    fixture.whenStable().then(() => {
      const element = fixture.elementRef.nativeElement;
      element.dispatchEvent(new Event('blur'));
      fixture.detectChanges();
      expect(touched).toBeTrue();
      done();
    });
  });

  it('updates inner value when ngModel is updated on the outside', () => {
    const date = new Date(Date.parse('01-01-2020 00:00:00'));
    const time = date.getTime();
    component.writeValue(date);
    expect(component.innerModel?.getTime()).toEqual(time);
  });

  it('makes ngModel undefined when set to not relevant', () => {
    const date = new Date(Date.parse('01-01-2020 00:00:00'));
    component.writeValue(date);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    expect(component.value).toBeUndefined();
  });

  it('re-fills the date when set to not relevant and back again', () => {
    const date = new Date(Date.parse('01-01-2020 00:00:00'));
    const time = date.getTime();
    component.writeValue(date);
    fixture.detectChanges();
    component.notRelevant = true;
    component.ngOnChanges({notRelevant: {currentValue: true, previousValue: undefined} as SimpleChange} as SimpleChanges);
    expect(component.value).toBeUndefined();
    component.notRelevant = false;
    component.ngOnChanges({notRelevant: {currentValue: false, previousValue: true} as SimpleChange} as SimpleChanges);
    expect(component.value?.getTime()).toEqual(time);
  });

  it('does not register a change on the first value set', () => {
    // This is to avoid issues where values initialized from an async call will mark the form as dirty
    const mock = {
      onChange: (whatever: any) => undefined
    };
    const onChange = spyOn(mock, 'onChange');
    const date = new Date(Date.parse('01-01-2020 00:00:00'));
    component.registerOnChange(onChange);
    component.writeValue(new Date());
    expect(onChange).not.toHaveBeenCalled();
    component.writeValue(date);
    expect(onChange).toHaveBeenCalledWith(date);
  });
});
