import {Component, EventEmitter, Input, OnChanges, OnInit, Output, SimpleChanges, ViewChild} from '@angular/core';
import {ModalComponent} from '../../modals/modal/modal.component';
import {
  CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest,
  DataCard,
  DataCardRequest
} from 'facade-api';
import {FormComponent} from '../../confirm-navigation-if-dirty.framework';
import {ControlContainer, NgForm} from '@angular/forms';
import {DataSourceService} from '../../services/data-source.service';
import {copy} from '../../../utils/copy';
import {merge, Observable} from 'rxjs';
import {finalize} from 'rxjs/operators';


type ObservationFormType = 'regular' | 'ultrasound';
@Component({
  selector: 'app-edit-measurements-modal',
  templateUrl: './edit-measurements-modal.component.html',
  styleUrls: ['./edit-measurements-modal.component.scss'],
  viewProviders: [ { provide: ControlContainer, useExisting: NgForm } ]
})
export class EditMeasurementsModalComponent extends FormComponent implements OnChanges, OnInit {
  public note = '';
  public isSaving = false;
  public dataCard?: DataCard;
  public observationData: CreateUltrasoundObservationsRequest|CreateRegularObservationsRequest = {fetuses: []};

  @ViewChild('modal') public modalComponent: ModalComponent | undefined;
  @ViewChild('form') public form: NgForm | undefined;
  @Input() public isOpen = false;
  @Input() public observationType: ObservationFormType = 'regular';
  @Input() public allowTypeChange = false;
  @Output() public modalClosed: EventEmitter<void> = new EventEmitter();
  private episodeOfCareId = '';

  constructor(private dataSourceService: DataSourceService) { super(); }

  ngOnInit(): void {
    this.dataSourceService.dummyDataCards$.subscribe((cards) => {
      const {identifier, episodeOfCareIdentifier, ...dataCard} = cards[0];
      this.dataCard = copy(dataCard);
      this.episodeOfCareId = episodeOfCareIdentifier;
      this.changeObservationType(this.observationType);
    });
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (changes.isOpen) {
      if (this.isOpen) {
        this.modalComponent?.setVisible(true);
      } else {
        this.handleClose();
      }
    }
  }

  public handleClose(): void {
    this.observationData.activityId = undefined;
    this.modalComponent?.setVisible(false);
    this.modalClosed.emit();
  }

  public validateActivityId(): boolean {
    const activitySelector = document.getElementById('activitySelector');
    if (!this.observationData.activityId && this.modalComponent?.isModalVisible) {
      activitySelector?.classList.add('ng-invalid');
      return false;
    }
    activitySelector?.classList.remove('ng-invalid');
    return true;
  }

  public changeObservationType(formType: ObservationFormType): void {
    this.observationData =  {
      activityId: this.observationData.activityId,
      fetuses: [...new Array(this.dataCard?.overallAssessmentSection?.prenatalCareAssessment?.numberOfFetuses || 1)].map((x, index) => ({id: `Foster ${index + 1}`}))
    };
  }

  public save(): void {
    if (!this.validateActivityId()) {
      const activitySelector = document.getElementById('activitySelector');
      activitySelector?.scrollIntoView({behavior: 'smooth', block: 'center'});
      return;
    }
    this.isSaving = true;
    const save = (this.observationType === 'ultrasound') ? this.saveUltrasound() : this.saveRegular();
    save// TODO: create note too (Eller hvad? Skal vi se om der allerede er en på aktiviteten og hvis ja redigere den?)
      .pipe(finalize(() => this.isSaving = false))
    .subscribe(() => {
      this.handleClose();
      this.dataSourceService.refresh();
    });
  }

  private saveUltrasound(): Observable<void> {
    return merge(
      this.dataSourceService.updateDataCard(this.episodeOfCareId, this.dataCard as DataCardRequest),
      this.dataSourceService.createUltrasoundObservations(this.observationData as CreateUltrasoundObservationsRequest)
    ) as unknown as Observable<void>;
  }

  private saveRegular(): Observable<void> {
    return this.dataSourceService.createRegularObservations(this.observationData as CreateRegularObservationsRequest);
  }
}
