import {Injectable} from '@angular/core';
import {BehaviorSubject, combineLatest, Observable, Subject} from 'rxjs';
import {
  Activity,
  ApplicationContext,
  CarePlan,
  CarePlanService, CreateRegularObservationsRequest,
  CreateUltrasoundObservationsRequest,
  DataCardRequest,
  DataCardResponse,
  DataCardService,
  Observation,
  ObservationService,
  Uuidv4
} from 'facade-api';
import {filter, map, shareReplay, switchMap} from 'rxjs/operators';
import {UserContextService} from './user-context.service';
import {HttpClient} from '@angular/common/http';

/**
 * Shared data service
 */
@Injectable({
  providedIn: 'root'
})
export class DataSourceService {
  // activate data Loading
  private innerDataLoadingActivator: Subject<undefined> = new BehaviorSubject(undefined);

  // load data when innerDataLoadingActivator is emitting and when role is selected
  private dataLoadingActivator: Observable<undefined> = combineLatest(
    [
      this.innerDataLoadingActivator,
      this.userContextService.contextIsInitialized$
      ]
  ).pipe(map(_ => undefined));
  private applicationContext: ApplicationContext = this.getCookie('application-context');

  constructor(
    private observationsService: ObservationService,
    private carePlanService: CarePlanService,
    private dataCardService: DataCardService,
    private userContextService: UserContextService,
    private httpClient: HttpClient
  ) {}

  public dummyDataCards$: Observable<Array<DataCardResponse>> = this.dataLoadingActivator.pipe(
    switchMap(() => this.dataCardService.maternityDataCardGet(this.applicationContext)),
    shareReplay(),
  );

  public carePlan$: Observable<Array<CarePlan>> = this.dataLoadingActivator.pipe(
    switchMap(() => this.carePlanService.maternityCarePlanGet(this.applicationContext)),
    shareReplay()
  );

  public activities$: Observable<Array<Activity>> = this.carePlan$.pipe(
    map((careplans) => careplans[0].activities || [])
  );

  public observations$: Observable<Array<Observation>> = this.dataLoadingActivator.pipe(
    switchMap(() => this.observationsService.maternityObservationGet(this.applicationContext)),
    shareReplay()
  );

  public getActivityById(id: string): Observable<Activity|null> {
    return this.carePlan$.pipe(
      map((careplans) => careplans[0].activities?.find(activity => activity.id === id) || null)
    );
  }

  public updateDataCard(identifier: Uuidv4, request: DataCardRequest): Observable<DataCardResponse> {
    return this.dataCardService.maternityDataCardIdentifierPut(identifier, request);
  }

  public createUltrasoundObservations(observations: CreateUltrasoundObservationsRequest): Observable<void> {
    return this.observationsService.maternityUltrasoundObservationsPost(this.applicationContext, observations);
  }

  public createRegularObservations(observations: CreateRegularObservationsRequest): Observable<void> {
    return this.observationsService.maternityRegularObservationsPost(this.applicationContext, observations);
  }

  public refresh(): void{
    this.innerDataLoadingActivator.next(undefined);
  }

  private getCookie<T>(name: string): T {
    const cookies: Array<string> = document.cookie.split(';');
    const numberOfCookies: number = cookies.length;
    const cookieName = `${name}=`;
    let c: string;

    for (let i = 0; i < numberOfCookies; i += 1) {
      c = cookies[i].replace(/^\s+/g, '');
      if (c.indexOf(cookieName) === 0) {
        return JSON.parse(c.substring(cookieName.length, c.length)) as T;
      }
    }
    return {} as T;
  }
}
