import {Injectable, OnDestroy} from '@angular/core';
import {HttpClient} from '@angular/common/http';
import {environment} from '../../environments/environment';
import {delay, map, shareReplay, tap} from 'rxjs/operators';
import {combineLatest, Observable, Subject, Subscription} from 'rxjs';
import {getLocalStorageUserContext, setLocalStorageOrganisationSor, setLocalStorageRole} from '../../utils/localstorage';
import {filterNullish} from '../../utils/rxjs-operators';
import {SorItem} from './sor.service';

export interface UserContext {
  commonName: string;
  organizationName: string;
  eligibleRoles: UserAuthorizationType[];
  patientIdentifier?: string;
  organizationSor?: SorItem;
}

export interface UserAuthorizationType {
  authorizationCode: string;
  educationCode: string;
  educationType: string;
}

export interface LocalStorageUserContext {
  role?: UserAuthorizationType;
  organizationSor?: SorItem;
}

@Injectable({
  providedIn: 'root'
})
export class UserContextService implements OnDestroy {

  private fetchUserContextLocalStorage: Subscription;
  private USER_CONTEXT_PATH = '/usercontext';
  private LOGOUT_PATH = '/saml/Logout';

  constructor(private http: HttpClient) {
    // Subscribe to make the observable hot so that any previous context will be loaded
    this.fetchUserContextLocalStorage =  this.userContext$.pipe(
      map(userContext => userContext.commonName),
      map(userName => getLocalStorageUserContext(userName))
      ).subscribe(
       localStorageContext => {
        if (localStorageContext?.role) {
          console.log('Fetched Role from localStorage: ' + JSON.stringify(localStorageContext.role));
          this.selectRole(localStorageContext.role);
        }
        if (localStorageContext?.organizationSor) {
          console.log('UserContextService Fetched organizationSor, localStorage:' + JSON.stringify(localStorageContext.organizationSor));
          this.selectOrganizationSor(localStorageContext.organizationSor);
        }
      }
    );
  }

  public userContext$: Observable<UserContext> = this.http.get<UserContext>(environment.base_path + '/api' + this.USER_CONTEXT_PATH).pipe(
    tap(userContext => {
    if (userContext.organizationSor){
      this.selectOrganizationSor(userContext.organizationSor);
    }
    }),
    delay(environment.mock ? 3000 : 0),
    shareReplay()
  );

  public eligibleRoles$: Observable<UserAuthorizationType[]> =
    this.userContext$.pipe(
    tap(userContext => {
      if (userContext.eligibleRoles && userContext.eligibleRoles.length === 1) {
        this.selectRole(userContext.eligibleRoles[0]);
      }
    }),
    map(userContext => userContext.eligibleRoles),
    shareReplay()
  );

  private innerSelectedRole: Subject<UserAuthorizationType> = new Subject();
  public selectedRole$: Observable<UserAuthorizationType> = combineLatest([
    this.userContext$.pipe(map(usercontext => usercontext.commonName)),
    this.innerSelectedRole.pipe(tap(role => console.log('selected role:' + JSON.stringify(role))))
  ]).pipe(
    tap(([userName, selectedRole]) => setLocalStorageRole(userName, selectedRole)),
    map(([_, selectedRole]) => selectedRole),
    shareReplay()
  );

  private innerSelectedOrganizationSor: Subject<SorItem> = new Subject();
  public selectedOrganizationSor$: Observable<SorItem> = combineLatest([
    this.userContext$.pipe(map(usercontext => usercontext.commonName)),
    this.innerSelectedOrganizationSor.pipe(tap(sor => console.log('selected organizationSor:' + sor)))
  ]).pipe(
    tap(([userName, selectedOrganizationSor]) => setLocalStorageOrganisationSor(userName, selectedOrganizationSor)),
    map(([_, selectedRole]) => selectedRole),
    shareReplay()
  );

  public contextIsInitialized$: Observable<boolean> = combineLatest([
    this.userContext$,
    this.selectedRole$,
    this.selectedOrganizationSor$
  ]).pipe(
  filterNullish(),
  map(result => true )
  );

  public logout(): void {
    console.log('Logout triggered');
    window.location.assign(environment.base_path + this.LOGOUT_PATH);
  }

  public selectRole( role: UserAuthorizationType): void {
    console.log('UserContextService selectRole() : ' + JSON.stringify(role));
    this.innerSelectedRole.next(role);
  }

  public selectOrganizationSor(organizationSor: SorItem): void {
    console.log('UserContextService selectOrganizationSor() : ' + JSON.stringify(organizationSor));
    this.innerSelectedOrganizationSor.next(organizationSor);
  }

  ngOnDestroy(): void {
    this.fetchUserContextLocalStorage.unsubscribe();
  }
}
