import { Directive, Input, ElementRef, HostListener, Renderer2 } from '@angular/core';
import {WindowService} from '../services/window.service';
import {DocumentService} from '../services/document.service';

@Directive({
  selector: '[appTooltip]'
})
export class TooltipDirective {
  @Input() appTooltip: string | undefined;
  @Input() placement = 'bottom';
  @Input() tooltipClasses: string | undefined;
  @Input() delay = 0;
  tooltip: HTMLElement | undefined;
  offset = 0;
  private timeout: number | undefined;
  private window: Window;
  private document: Document;

  constructor(private el: ElementRef,
              private renderer: Renderer2,
              private windowService: WindowService,
              private documentService: DocumentService) {
    this.window = windowService.nativeWindow();
    this.document = documentService.nativeDocument();
  }

  @HostListener('mouseenter') onMouseEnter(): void {
    if (!this.tooltip) {
      this.timeout = this.window.setTimeout(() => {
        this.show();
      }, this.delay);
    }
  }

  @HostListener('mouseleave') onMouseLeave(): void {
    if (this.timeout) { clearTimeout(this.timeout); }
    if (this.tooltip) { this.hide(); }
  }

  public show(): void {
    this.create();
    this.setPosition();
    this.renderer.addClass(this.tooltip, 'ng-tooltip-show');
  }

  public hide(): void {
    this.renderer.removeClass(this.tooltip, 'ng-tooltip-show');
    this.window.setTimeout(() => {
      this.renderer.removeChild(this.el.nativeElement, this.tooltip);
      this.tooltip = undefined;
    }, 500);
  }

  create(): void {
    this.tooltip = this.renderer.createElement('span');

    if (this.appTooltip != null) {
      this.renderer.appendChild(
        this.tooltip,
        this.renderer.createText(this.appTooltip)
      );
    }

    this.renderer.appendChild(this.el.nativeElement, this.tooltip);
    this.renderer.addClass(this.tooltip, 'ng-tooltip');
    this.renderer.addClass(this.tooltip, `ng-tooltip-${this.placement}`);
    this.renderer.setStyle(this.tooltip, '-webkit-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, '-moz-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, '-o-transition', `opacity ${this.delay}ms`);
    this.renderer.setStyle(this.tooltip, 'transition', `opacity ${this.delay}ms`);
    if (this.tooltipClasses !== undefined) {
      this.renderer.addClass(this.tooltip, this.tooltipClasses);
    }
  }

  setPosition(): void {
    if (this.tooltip === undefined) {
      return;
    }
    const hostPos = this.el.nativeElement.getBoundingClientRect();
    const tooltipPos = this.tooltip.getBoundingClientRect();
    const scrollPos = this.window.pageYOffset || this.document.documentElement.scrollTop || this.document.body.scrollTop || 0;
    let top;
    let left;

    if (this.placement === 'top') {
      top = hostPos.top - tooltipPos.height - this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2 - (tooltipPos.width / 2);
    } else if (this.placement === 'left') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.left - tooltipPos.width - this.offset;
      this.renderer.setStyle(this.tooltip, 'border-radius', `4px`);
    } else if (this.placement === 'right') {
      top = hostPos.top + (hostPos.height - tooltipPos.height) / 2;
      left = hostPos.right + this.offset;
      this.renderer.setStyle(this.tooltip, 'border-radius', `4px`);
    } else {
      top = hostPos.bottom + this.offset;
      left = hostPos.left + (hostPos.width - tooltipPos.width) / 2 - (tooltipPos.width / 2);
    }

    console.log('positions:', top, left);
    this.renderer.setStyle(this.tooltip, 'top', `${top + scrollPos}px`);
    this.renderer.setStyle(this.tooltip, 'left', `${left}px`);
  }
}
