import {HttpErrorResponse, HttpInterceptor, HttpRequest, HttpResponse} from '@angular/common/http';
import {HttpRequestInterceptor} from './interceptor';
import {ActiveRequests} from '../app/services/active-requests.service';
import {of, throwError} from 'rxjs';
import {fakeAsync, flush} from '@angular/core/testing';
import {AppError} from './errorHandling';

describe('Http Interceptor', () => {
  let service: HttpInterceptor;
  let activeRequests: ActiveRequests;

  beforeEach(() => {
    activeRequests = {
      _registerBlockingRequest: () => undefined,
      _unregisterBlockingRequest: () => undefined,
    } as ActiveRequests;
    service = new HttpRequestInterceptor(activeRequests);
  });

  it('increments active requests when intercepting', () => {
    const activeRequestService = spyOn(activeRequests, '_registerBlockingRequest').and.callThrough();
    service.intercept({} as HttpRequest<any>, {handle: () => of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}))});
    expect(activeRequestService).toHaveBeenCalledTimes(1);
  });

  it('decrements active requests when  request finishes', fakeAsync(() => {
    const unregisterBlockingRequest = spyOn(activeRequests, '_unregisterBlockingRequest').and.callThrough();
    service.intercept({} as HttpRequest<any>, {handle: () => of(new HttpResponse({body: {}, status: 200, statusText: 'OK'}))})
      .subscribe(() => undefined);
    flush();
    expect(unregisterBlockingRequest).toHaveBeenCalledTimes(1);
  }));

  it('decrements active requests even if a request errors', fakeAsync(() => {
    const error = new HttpErrorResponse({error: 'oops', status: 500 });
    const unregisterBlockingRequest = spyOn(activeRequests, '_unregisterBlockingRequest').and.callThrough();
    service.intercept({} as HttpRequest<any>, {handle: () => throwError(error)})
      .subscribe(() => undefined);
    expect(() => flush()).toThrow(new AppError(error));
    expect(unregisterBlockingRequest).toHaveBeenCalledTimes(1);
  }));

  it('transforms date strings into js Date-objects', (done) => {
    service.intercept(
      {} as HttpRequest<any>,
      {handle: () => of(new HttpResponse({body: {whateverDate: '2020-11-08T11:06:15.31'}, status: 200, statusText: 'OK'}))}
    ).subscribe((event) => {
      expect(typeof (event as any).body.whateverDate).toBe('object');
      expect((event as any).body.whateverDate.toDateString()).toEqual('Sun Nov 08 2020');
      done();
    });

  });
});
